use crate::util::surrealdb::schemas;
use leptos::*;
use leptos_use::*;

use super::ParticipantsAction;

pub fn init_websocket() {
    let UseWebsocketReturn {
        ready_state,
        message,
        ..
    } = use_websocket("ws://localhost:3000/ws");

    provide_context(ready_state);

    create_effect(move |prev_value| {
        let msg = message.get();

        if prev_value != Some(msg.clone()) {
            match msg {
                Some(ref text) => handle_message(text),
                None => (),
            }
        }

        msg
    });
}

fn handle_message(message: &str) {
    let action: ParticipantsAction = match serde_json::from_str(message) {
        Ok(res) => res,
        Err(_err) => {
            return;
        }
    };

    logging::log!("Recieved action: {:?}", action);

    let participants_context = use_context::<schemas::ParticipantsContext>().unwrap();

    match action {
        ParticipantsAction::Add { participant } => participants_context.update(|participants| {
            participants.push(schemas::ParticipantSignal {
                id: participant.id.clone(),
                value: create_rw_signal(participant),
            })
        }),
        ParticipantsAction::Remove { id } => participants_context
            .update(|participants| participants.retain(|participant| participant.id != id)),
        ParticipantsAction::Replace { participant } => {
            participants_context.with(|participants| {
                for participant_signal in participants {
                    if participant_signal.id == participant.id {
                        participant_signal.value.update(|value| {
                            *value = participant.clone();
                        })
                    }
                }
            });
        }
    }
}
